import pandas as pd
import numpy as np 
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
import csv
import copy
from sklearn.model_selection import train_test_split  
from sklearn.metrics import classification_report, confusion_matrix 

Pitch = []
Roll = []
Yaw = []
X = []
Y = []
Z = []
Class = []
colmap = {1:'red',2:'green',3:'blue'}
#k is number of activites
k = 3

def createGraph(df,centroids,colmap,improved):
	fig = plt.figure()
	ax = fig.gca(projection='3d')
	if improved:
		ax.scatter(df['x'],df['y'],df['z'], color=df['color'], alpha=0.5, edgecolor='k')
	else:
		ax.scatter(df['x'],df['y'],df['z'], color ='k')

	for i in centroids.keys():
		plt.scatter(*centroids[i],color = colmap[i])
	
	ax.set_zlim3d(0, max(df['z']))
	ax.set_ylim3d(0, max(df['y']))                
	ax.set_xlim3d(0, max(df['x'])) 
	ax.set_xlabel( 'X' )
	ax.set_ylabel( 'Y' )
	ax.set_zlabel( 'Z' )
	plt.show()

def assignment(df,centroids,colmap):
	for i in centroids.keys():
	        # sqrt((x1 - x2)^2 - (y1 - y2)^2) - (z1 - z1)^2)
	        df['distance_from_{}'.format(i)] = (
	            np.sqrt(
	                (df['x'] - centroids[i][0]) ** 2
	                + (df['y'] - centroids[i][1]) ** 2
	                + (df['z'] - centroids[i][2]) ** 2
	            )
	        )
	centroid_distance_cols = ['distance_from_{}'.format(i) for i in centroids.keys()]
	df['closest'] = df.loc[:, centroid_distance_cols].idxmin(axis=1)
	df['closest'] = df['closest'].map(lambda x: int(x.lstrip('distance_from_')))
	df['color'] = df['closest'].map(lambda x: colmap[x])
	return df	

def changeofDegree(v1,v2):
	if abs((float(v1) - float(v2))) < 180:
		return abs((float(v1) - float(v2)))
	else:
		return 360 - abs((float(v1) - float(v2)))

def importChangeofData(csvname,classInput):
	with open(csvname) as csvfile:
		reader = csv.reader(csvfile)
		row = reader.next()
		while True:
			if(len(row) > 1):	
				try:
					cur_row = reader.next()
					Pitch.append(changeofDegree(cur_row[0],row[0]))
					Roll.append(changeofDegree(cur_row[1],row[1]))
					Yaw.append(changeofDegree(cur_row[2],row[2]))
					X.append(changeofDegree(cur_row[3],row[3]))
					Y.append(changeofDegree(cur_row[4],row[4]))
					Z.append(changeofDegree(cur_row[5],row[5]))
					Class.append(classInput)
					row = cur_row
				except:
					break

def update(df,centroids):
	
	for i in centroids.keys():
		centroids[i][0] = np.mean(df[df['closest'] == i]['x'])
		centroids[i][1] = np.mean(df[df['closest'] == i]['y'])
		centroids[i][2] = np.mean(df[df['closest'] == i]['z'])
	return centroids
		
importChangeofData('stilltestdata.csv',1)
importChangeofData('newStanding_edit.csv',1)
importChangeofData('10minRun1.csv',3)
importChangeofData('10minRun2.csv',3)
importChangeofData('runtestdata.csv',3)
importChangeofData('leftLegWalk.csv',2)
importChangeofData('RightLegWalk.csv',2)
importChangeofData('walktestdata.csv',2)
importChangeofData('WalkinginCircles.csv',2)

df = pd.DataFrame({
	'x': X,
	'y':Y,
	'z':Z,
	'Pitch': Pitch,
	'Roll': Roll,
	'Yaw': Yaw
	})

centroids = {
	i+1: [max(df['x']) * np.random.random_sample(),max(df['y']) * np.random.random_sample(),np.random.random_sample() * max(df['z'])]
	for i in range(k)
}


#createGraph(df,centroids,colmap,False)
df = assignment(df,centroids,colmap)
centroids = update(df,centroids)
df = assignment(df, centroids,colmap)
while True:
    closest_centroids = df['closest'].copy(deep=True)
    centroids = update(df,centroids)
    df = assignment(df, centroids,colmap)
    if closest_centroids.equals(df['closest']):
    	#createGraph(df,centroids,colmap,True)
    	break
print(confusion_matrix(Class,df['closest']))  
print(classification_report(Class,df['closest']))   
